#include "AssignIPDlg.h"
#include "IPConfigureFrame.h"
#include "DeviceListCtrl.h"
#include <algorithm>
#include <apps/Common/Info.h>
#include <apps/Common/mvIcon.xpm>
#include <apps/Common/wxAbstraction.h>
#include <string>
#include "TLILibImports.h"
#include "wx/config.h"
#include <wx/splitter.h>
#include <wx/spinctrl.h>

using namespace mvIMPACT::acquire;
using namespace std;

//-----------------------------------------------------------------------------
/// \brief A automatically managed buffer type to handle subscriptable data.
template<class T>
class auto_array_ptr
//-----------------------------------------------------------------------------
{
	size_t	m_cnt;
	T*		m_pBuf;
public:
	auto_array_ptr( size_t initial_size = 0 ) : m_cnt(initial_size), m_pBuf(0)
	{
		if( initial_size > 0 )
		{
			m_pBuf = new T[initial_size];
		}
	}
	/// \brief Pass ownership to the new object.
	auto_array_ptr( auto_array_ptr& rhs ) : m_cnt(rhs.m_cnt), m_pBuf(rhs.release()) {}
	/// \brief Pass ownership to the new object.
	///
	/// Do NOT delete the original T*. This is done by this class now.
	auto_array_ptr( T* pBuf, size_t cnt ) : m_cnt(cnt), m_pBuf(pBuf) {}
	~auto_array_ptr()
	{
		delete [] m_pBuf;
	}
	// functions
	operator T*() { return m_pBuf; }
	/// \brief Pass ownership. Old buffer of the left hand side object is
	/// freed, the lhs object takes ownership of the buffer of the rhs object.
	auto_array_ptr& operator=( auto_array_ptr& rhs )
	{
		if( this != &rhs )
		{
			delete [] m_pBuf;
			m_cnt = rhs.m_cnt;
			m_pBuf = rhs.release();
		}
		return *this;
	}
	/// \brief free old buffer, allocate new buffer
	void realloc( size_t newsize )
	{
		if( newsize > 0 )
		{
			delete [] m_pBuf;
			m_pBuf = new T[newsize];
			m_cnt = newsize;
		}
	}
	/// \brief increases the buffer size and keeps the old data
	void grow( size_t newsize )
	{
		if( newsize > m_cnt )
		{
			T* p = new T[newsize];
			memcpy( p, m_pBuf, m_cnt );
			delete [] m_pBuf;
			m_pBuf = p;
			m_cnt = newsize;
		}
	}
	/// \brief Release ownership, return pointer to buffer.
	T* release( void ) { T* p = m_pBuf; m_pBuf = 0; m_cnt = 0; return p; }
	/// \brief Return pointer to buffer.
	T* get( void ) { return m_pBuf; }
	/// \brief Return const pointer to buffer.
	const T* get( void ) const { return m_pBuf; }
	/// \brief Return element count.
	size_t parCnt( void ) const { return m_cnt; }
};

//-----------------------------------------------------------------------------
template<class _Ty1, class _Ty2>
void DeleteSecond( std::pair<_Ty1, _Ty2>& data )
//-----------------------------------------------------------------------------
{
	delete data.second;
	data.second = 0;
}

//-----------------------------------------------------------------------------
class MyApp : public wxApp
//-----------------------------------------------------------------------------
{
public:
	virtual bool OnInit();
};

// ----------------------------------------------------------------------------
// event tables and other macros for wxWidgets
// ----------------------------------------------------------------------------

// the event tables connect the wxWidgets events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.
BEGIN_EVENT_TABLE(IPConfigureFrame, wxFrame)
	EVT_MENU(miAbout, IPConfigureFrame::OnAbout)
	EVT_MENU(miQuit, IPConfigureFrame::OnQuit)
	EVT_MENU(miAction_AssignTemporaryIP, IPConfigureFrame::OnAssignTemporaryIP)
	EVT_MENU(miAction_UpdateDeviceList, IPConfigureFrame::OnUpdateDeviceList)
	EVT_TEXT(widInterfaceSelector, IPConfigureFrame::OnInterfaceSelectorTextChanged)
	EVT_TEXT(widPersistentIPAddress, IPConfigureFrame::OnPersistentIPTextChanged)
	EVT_TEXT(widPersistentSubnetMask, IPConfigureFrame::OnPersistentNetmaskTextChanged)
	EVT_TEXT(widPersistentDefaultGateway, IPConfigureFrame::OnPersistentGatewayTextChanged)
	EVT_BUTTON(widBtnApplyChanges, IPConfigureFrame::OnBtnApplyChanges)
	EVT_BUTTON(widBtnConfigure, IPConfigureFrame::OnBtnConfigure)
	EVT_CHECKBOX(widCBUsePersistentIP, IPConfigureFrame::OnCBUsePersistentIP)
	EVT_CHECKBOX(widCBUseDHCP, IPConfigureFrame::OnCBUseDHCP)
END_EVENT_TABLE()

// Create a new application object: this macro will allow wxWidgets to create
// the application object during program execution (it's better than using a
// static object for many reasons) and also declares the accessor function
// wxGetApp() which will return the reference of the right type (i.e. MyApp and
// not wxApp)
IMPLEMENT_APP(MyApp)

//-----------------------------------------------------------------------------
// `Main program' equivalent: the program execution "starts" here
bool MyApp::OnInit()
//-----------------------------------------------------------------------------
{
	// Create the main application window
	IPConfigureFrame *frame = new IPConfigureFrame( wxString::Format( wxT("Configuration tool for GigE Vision(tm) devices(%s)"), VERSION_STRING ), wxDefaultPosition, wxDefaultSize );
	frame->Show(true);
	SetTopWindow(frame);
	// success: wxApp::OnRun() will be called which will enter the main message
	// loop and the application will run. If we returned false here, the
	// application would exit immediately.
	return true;
}

//=============================================================================
//================= Implementation FramePositionStorage =======================
//=============================================================================
//-----------------------------------------------------------------------------
class FramePositionStorage
//-----------------------------------------------------------------------------
{
	wxConfigBase* m_pConfig;
	wxWindow* m_pWin;
public:
	FramePositionStorage( wxWindow* pWin ) : m_pConfig(wxConfigBase::Get()), m_pWin(pWin) {}
	void store( void ) const
	{
		int Height, Width, XPos, YPos;
		m_pWin->GetSize( &Width, &Height );
		m_pWin->GetPosition( &XPos, &YPos );
		m_pConfig->Write( wxT("/MainFrame/h"), Height );
		m_pConfig->Write( wxT("/MainFrame/w"), Width );
		m_pConfig->Write( wxT("/MainFrame/x"), XPos );
		m_pConfig->Write( wxT("/MainFrame/y"), YPos );
	}
	wxRect restore( const wxRect& defaultDimensions ) const
	{
		wxRect rect;
		rect.height = m_pConfig->Read( wxT("/MainFrame/h"), defaultDimensions.height );
		rect.width = m_pConfig->Read( wxT("/MainFrame/w"), defaultDimensions.width );
		rect.x = m_pConfig->Read( wxT("/MainFrame/x"), defaultDimensions.x );
		rect.y = m_pConfig->Read( wxT("/MainFrame/y"), defaultDimensions.y );
		int displayWidth = 0;
		int displayHeight = 0;
		wxDisplaySize( &displayWidth, &displayHeight );
		if( ( rect.x >= displayWidth ) || ( rect.x < 0 ) )
		{
			rect.x = 0;
		}
		if( ( rect.y >= displayHeight ) || ( rect.y < 0 ) )
		{
			rect.y = 0;
		}
		return rect;
	}
};

//=============================================================================
//================= Implementation IPStringValidator ==========================
//=============================================================================
//-----------------------------------------------------------------------------
IPStringValidator::IPStringValidator( wxString* valPtr /* = NULL */ ) : wxTextValidator(wxFILTER_INCLUDE_CHAR_LIST, valPtr)
//-----------------------------------------------------------------------------
{
	wxArrayString strings;
	strings.push_back( wxT(".") );
	for( unsigned int i=0; i<=9; i++ )
	{
		wxString s;
		s << i;
		strings.push_back( s );
	}
	SetIncludes( strings );
}

//=============================================================================
//================= Implementation IPConfigureFrame ===========================
//=============================================================================
const wxTextAttr IPConfigureFrame::m_ERROR_STYLE(wxColor(255, 0, 0));

//-----------------------------------------------------------------------------
IPConfigureFrame::IPConfigureFrame( const wxString& title, const wxPoint& pos, const wxSize& size )
	: wxFrame((wxFrame *)NULL, wxID_ANY, title, pos, size), m_pLogWindow(0), m_pTCPersistentIPAddress(0), 
	m_pTCPersistentSubnetMask(0), m_pTCPersistentDefaultGateway(0), m_hTLI(0)
//-----------------------------------------------------------------------------
{
	wxMenu *menuAction = new wxMenu;
	m_pMIActionAssignTemporaryIP = menuAction->Append( miAction_AssignTemporaryIP, wxT("&Assign temporary IP\tCTRL+C") );

	menuAction->AppendSeparator();
	m_pMIActionUpdateDeviceList = menuAction->Append( miAction_UpdateDeviceList, wxT("&Update device list\tALT+CTRL+U") );

	menuAction->AppendSeparator();
	menuAction->Append( miQuit, wxT("E&xit\tALT+X"));

	wxMenu *menuHelp = new wxMenu;
	menuHelp->Append( miAbout, wxT("&About mvIPConfigure"));

	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append( menuAction, wxT("&Action") );
	menuBar->Append( menuHelp, wxT("&Help") );
	// ... and attach this menu bar to the frame
	SetMenuBar(menuBar);

	// define the applications icon
	wxIcon icon(mvIcon_xpm);
	SetIcon( icon );

	wxPanel* pPanel = new wxPanel(this);

	// splitter for log window and the set of controls in the upper part of the dialog
	m_pHorizontalSplitter = new wxSplitterWindow( pPanel, widHorSplitter, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER );
	m_pHorizontalSplitter->SetMinimumPaneSize( 25 );

	// splitter for device list and the device info window
	m_pVerticalSplitter = new wxSplitterWindow( m_pHorizontalSplitter, widVerSplitter, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER );
	m_pVerticalSplitter->SetMinimumPaneSize( 270 );

	// device list on the left side of the splitter
	m_pDevListCtrl = new DeviceListCtrl( m_pVerticalSplitter, LIST_CTRL, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_SINGLE_SEL | wxSUNKEN_BORDER, this );
	m_pDevListCtrl->InsertColumn( lcProduct, wxT("product") );
	m_pDevListCtrl->InsertColumn( lcSerial, wxT("serial") );

	// and a new panel for the device info and controls on the right
	wxScrolledWindow* pControlsPanel = new wxScrolledWindow(m_pVerticalSplitter);
	pControlsPanel->SetScrollRate( 10, 10 );

	const int GROUPBOX_BORDER_WIDTH_PIXEL = 5;
	const int BTN_BORDER_WIDTH_PIXEL = 4;
	const int CHECKBOX_BORDER_PIXEL_WIDTH = 3;
	
	// device information controls
	wxBoxSizer* pDeviceInfoSizer = new wxStaticBoxSizer(wxVERTICAL, pControlsPanel, wxT("Device information: "));
	wxFlexGridSizer* pDeviceInfoElementsGridSizer = new wxFlexGridSizer(2, 4);
	pDeviceInfoElementsGridSizer->AddGrowableCol( 1, 3 );

	// row 1
	pDeviceInfoElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("Manufacturer: ")), wxSizerFlags().Left() );
	m_pSTManufacturer = new wxStaticText(pControlsPanel, widDeviceManufacturer, wxT("-"), wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE);
	m_pSTManufacturer->Wrap(-1);
	pDeviceInfoElementsGridSizer->Add( m_pSTManufacturer, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 2
	pDeviceInfoElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("Serial number: ")), wxSizerFlags().Left() );
	m_pSTSerialNumber = new wxStaticText(pControlsPanel, widDeviceSerial, wxT("-"));
	pDeviceInfoElementsGridSizer->Add( m_pSTSerialNumber, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 3
	pDeviceInfoElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("User defined name: ")), wxSizerFlags().Left() );
	m_pTCUserDefinedName = new wxTextCtrl(pControlsPanel, widUserDefinedName);
	pDeviceInfoElementsGridSizer->Add( m_pTCUserDefinedName, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 4
	pDeviceInfoElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("Interface count: ")), wxSizerFlags().Left() );
	m_pSTInterfaceCount = new wxStaticText(pControlsPanel, widDeviceInterfaceCount, wxT("-"));
	pDeviceInfoElementsGridSizer->Add( m_pSTInterfaceCount, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );

	pDeviceInfoSizer->Add( pDeviceInfoElementsGridSizer, wxSizerFlags().Align( wxGROW ) );

	// interface selector
	wxBoxSizer* pInterfaceSelector = new wxBoxSizer(wxHORIZONTAL);
	pInterfaceSelector->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("Selected interface:") ));
	m_pSCInterfaceSelector = new wxSpinCtrl(pControlsPanel, widInterfaceSelector, wxT("0"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0, 3, 0);
	pInterfaceSelector->Add( m_pSCInterfaceSelector, wxSizerFlags().Left() );

	// current IP address controls
	wxBoxSizer* pCurrentIPSizer = new wxStaticBoxSizer(wxVERTICAL, pControlsPanel, wxT("Current interface parameter: "));
	// device information controls
	wxFlexGridSizer* pCurrentIPElementsGridSizer = new wxFlexGridSizer(2, 5);
	pCurrentIPElementsGridSizer->AddGrowableCol( 1, 3 );

	// row 1
	pCurrentIPElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("IP address: ")), wxSizerFlags().Left() );
	m_pSTCurrentIPAddress = new wxStaticText(pControlsPanel, widCurrentIPAddress, wxT("-"));
	pCurrentIPElementsGridSizer->Add( m_pSTCurrentIPAddress, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 2
	pCurrentIPElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("Subnet mask: ")), wxSizerFlags().Left() );
	m_pSTCurrentSubnetMask = new wxStaticText(pControlsPanel, widCurrentSubnetMask, wxT("-"));
	pCurrentIPElementsGridSizer->Add( m_pSTCurrentSubnetMask, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 3
	pCurrentIPElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("Default gateway: ")), wxSizerFlags().Left() );
	m_pSTCurrentDefaultGateway = new wxStaticText(pControlsPanel, widCurrentDefaultGateway, wxT("-"));
	pCurrentIPElementsGridSizer->Add( m_pSTCurrentDefaultGateway, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 4
	pCurrentIPElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("MAC address: ")), wxSizerFlags().Left() );
	m_pSTMACAddress = new wxStaticText(pControlsPanel, widInterfaceMACAddress, wxT("-"));
	pCurrentIPElementsGridSizer->Add( m_pSTMACAddress, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 5
	pCurrentIPElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("Connected to IP address: ")), wxSizerFlags().Left() );
	m_pSTConnectedToIPAddress = new wxStaticText(pControlsPanel, widConnectToIPAddress, wxT("-"));
	pCurrentIPElementsGridSizer->Add( m_pSTConnectedToIPAddress, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	pCurrentIPSizer->Add( pCurrentIPElementsGridSizer, wxSizerFlags().Align( wxGROW ) );

	// persistent IP address related controls
	wxBoxSizer* pPersistentIPSizer = new wxStaticBoxSizer(wxVERTICAL, pControlsPanel, wxT("Persistent IP address: "));
	wxFlexGridSizer* pPersistentIPEditElementsGridSizer = new wxFlexGridSizer(2, 3);
	pPersistentIPEditElementsGridSizer->AddGrowableCol( 1, 3 );

	// row 1
	pPersistentIPEditElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("IP address: ")), wxSizerFlags().Left() );
	m_pTCPersistentIPAddress = new wxTextCtrl(pControlsPanel, widPersistentIPAddress, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, m_IPStringValidator);
	pPersistentIPEditElementsGridSizer->Add( m_pTCPersistentIPAddress, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 2
	pPersistentIPEditElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("Subnet mask: ")), wxSizerFlags().Left() );
	m_pTCPersistentSubnetMask = new wxTextCtrl(pControlsPanel, widPersistentSubnetMask, wxT("255.255.255.0"), wxDefaultPosition, wxDefaultSize, 0, m_IPStringValidator);
	pPersistentIPEditElementsGridSizer->Add( m_pTCPersistentSubnetMask, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 3
	pPersistentIPEditElementsGridSizer->Add( new wxStaticText(pControlsPanel, wxID_ANY, wxT("Default gateway: ")), wxSizerFlags().Left() );
	m_pTCPersistentDefaultGateway = new wxTextCtrl(pControlsPanel, widPersistentDefaultGateway, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, m_IPStringValidator);
	pPersistentIPEditElementsGridSizer->Add( m_pTCPersistentDefaultGateway, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );

	pPersistentIPSizer->Add( pPersistentIPEditElementsGridSizer, wxSizerFlags().Align( wxGROW ) );

	// IP configuration related controls
	wxBoxSizer* pIPConfigurationSizer = new wxStaticBoxSizer(wxVERTICAL, pControlsPanel, wxT("IP configuration: "));
	m_pCBUsePersistentIP = new wxCheckBox(pControlsPanel, widCBUsePersistentIP, wxT("use persistent IP"));
	pIPConfigurationSizer->Add( m_pCBUsePersistentIP, wxSizerFlags().Expand().Border( wxALL, CHECKBOX_BORDER_PIXEL_WIDTH ) );
	m_pCBUseDHCP = new wxCheckBox(pControlsPanel, widCBUseDHCP, wxT("use DHCP"));
	pIPConfigurationSizer->Add( m_pCBUseDHCP, wxSizerFlags().Expand().Border( wxALL, CHECKBOX_BORDER_PIXEL_WIDTH ) );
	m_pCBUseLLA = new wxCheckBox(pControlsPanel, wxID_ANY, wxT("use LLA"));
	pIPConfigurationSizer->Add( m_pCBUseLLA, wxSizerFlags().Expand().Border( wxALL, CHECKBOX_BORDER_PIXEL_WIDTH ) );
	m_pCBUseLLA->Enable( false );
	m_pCBUseLLA->SetValue( true );

	// interface configuration controls
	wxBoxSizer* pInterfaceConfigurationSizer = new wxStaticBoxSizer(wxVERTICAL, pControlsPanel, wxT("Interface configuration: "));
	pInterfaceConfigurationSizer->Add( pInterfaceSelector, wxSizerFlags().Expand().Border( wxALL, GROUPBOX_BORDER_WIDTH_PIXEL ) );
	pInterfaceConfigurationSizer->Add( pCurrentIPSizer, wxSizerFlags().Expand().Border( wxALL, GROUPBOX_BORDER_WIDTH_PIXEL ) );
	pInterfaceConfigurationSizer->Add( pPersistentIPSizer, wxSizerFlags().Expand().Border( wxALL, GROUPBOX_BORDER_WIDTH_PIXEL ) );
	pInterfaceConfigurationSizer->Add( pIPConfigurationSizer, wxSizerFlags().Expand().Border( wxALL, GROUPBOX_BORDER_WIDTH_PIXEL ) );

	wxBoxSizer* pButtonSizer = new wxBoxSizer(wxHORIZONTAL);
	m_pBtnConfigure = new wxButton(pControlsPanel, widBtnConfigure, wxT("&Configure"));
	pButtonSizer->Add( m_pBtnConfigure, wxSizerFlags().Right().Border( wxALL, BTN_BORDER_WIDTH_PIXEL ) );
	m_pBtnApplyChanges = new wxButton(pControlsPanel, widBtnApplyChanges, wxT("&Apply changes"));
	pButtonSizer->Add( m_pBtnApplyChanges, wxSizerFlags().Right().Border( wxALL, BTN_BORDER_WIDTH_PIXEL ) );

	wxBoxSizer* pControlsSizer = new wxBoxSizer(wxVERTICAL);
	pControlsSizer->Add( pDeviceInfoSizer, wxSizerFlags().Expand().Border( wxALL, GROUPBOX_BORDER_WIDTH_PIXEL ) );
	pControlsSizer->Add( pInterfaceConfigurationSizer, wxSizerFlags().Expand().Border( wxALL, GROUPBOX_BORDER_WIDTH_PIXEL ) );
	pControlsSizer->Add( pButtonSizer, wxSizerFlags().Right() );
	pControlsSizer->AddSpacer( 25 );

	m_pLogWindow = new wxTextCtrl(m_pHorizontalSplitter, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxSUNKEN_BORDER | wxTE_RICH | wxTE_READONLY);
	
	wxBoxSizer* pSizer = new wxBoxSizer(wxVERTICAL);
	pSizer->Add( m_pHorizontalSplitter, wxSizerFlags(1).Expand() );

	wxRect defaultRect(0, 0, 640, 480);
	pPanel->SetSizer( pSizer );
	// restore previous state
	wxConfigBase* pConfig = wxConfigBase::Get();
	FramePositionStorage frameStorage(this);
	wxRect rect = frameStorage.restore( defaultRect );
	int verticalSplitterPos = pConfig->Read( wxT("/MainFrame/verticalSplitter"), -1l );
	int horizontalSplitterPos = pConfig->Read( wxT("/MainFrame/horizontalSplitter"), -1l );
	
	m_pVerticalSplitter->SplitVertically( m_pDevListCtrl, pControlsPanel, 0 );
	m_pHorizontalSplitter->SplitHorizontally( m_pVerticalSplitter, m_pLogWindow, 0 );

	pControlsPanel->SetSizer( pControlsSizer );
	SetClientSize( pSizer->GetMinSize() );
	pSizer->SetSizeHints( this );
	SetSize( rect );

	m_pVerticalSplitter->SetSashPosition( ( verticalSplitterPos != -1 ) ? verticalSplitterPos : rect.width/3, true );
	m_pHorizontalSplitter->SetSashPosition( ( horizontalSplitterPos != -1 ) ? horizontalSplitterPos : 2*rect.height/3, true );

	int status = 0;
	LOGGED_TLI_CALL( TLOpen, ( &m_hTLI ), WriteLogMessage )
	UpdateDeviceList();
}

//-----------------------------------------------------------------------------
IPConfigureFrame::~IPConfigureFrame()
//-----------------------------------------------------------------------------
{
	// store the current state of the application
	FramePositionStorage frameStorage(this);
	frameStorage.store();
	wxConfigBase* pConfig = wxConfigBase::Get();
	pConfig->Write( wxT("/MainFrame/verticalSplitter"), m_pVerticalSplitter->GetSashPosition() );
	pConfig->Write( wxT("/MainFrame/horizontalSplitter"), m_pHorizontalSplitter->GetSashPosition() );
	InterfaceContainer::iterator it = m_TLIInterfaces.begin();
	InterfaceContainer::iterator itEnd = m_TLIInterfaces.end();
	int status = 0;
	while( it != itEnd )
	{
		LOGGED_TLI_CALL( TLCloseInterface, ( it->second ), WriteLogMessage )
		++it;
	}
	TLClose( m_hTLI );
	for_each( m_devices.begin(), m_devices.end(), ptr_fun(DeleteSecond<const string, DetecedDeviceInfo*>) );
	m_devices.clear();
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::BuildList( void )
//-----------------------------------------------------------------------------
{
	m_pDevListCtrl->DeleteAllItems();
	DeviceMap::const_iterator it = m_devices.begin();
	DeviceMap::const_iterator itEND = m_devices.end();
	long devCount=0;
	while( it != itEND )
	{
		long index = m_pDevListCtrl->InsertItem( devCount, ConvertedString(it->second->modelName_) );
		m_pDevListCtrl->SetItem( index, lcSerial, ConvertedString(it->first) );
		++it;
		++devCount;
	}

	for( unsigned int i=0; i<lcLAST_COLUMN; i++ )
	{
		m_pDevListCtrl->SetColumnWidth( i, ( ( devCount == 0 ) ? wxLIST_AUTOSIZE_USEHEADER : wxLIST_AUTOSIZE ) );
	}
	UpdateDlgControls( false );
}

//-----------------------------------------------------------------------------
std::string IPConfigureFrame::GetDeviceInterfaceStringInfo( MVTLI_INTERFACE_HANDLE hInterface, const std::string& deviceName, unsigned int interfaceIndex, DEVICE_INFO_CMD info )
//-----------------------------------------------------------------------------
{
	size_t stringSize = 0;
	int result = TLIMV_TLGetDeviceInterfaceParam( hInterface, deviceName.c_str(), interfaceIndex, info, 0, 0, &stringSize );
	if( result != 0 )
	{
		WriteLogMessage( wxString::Format( wxT("ERROR during call to TLIMV_TLGetDeviceInterfaceParam( %p, %s, %d, %d, 0, 0, %p ): %d.\n"), hInterface, deviceName.c_str(), interfaceIndex, info, &stringSize, result ), m_ERROR_STYLE );
		return string("");
	}
	auto_array_ptr<char> pStringBuffer(stringSize);
	result = TLIMV_TLGetDeviceInterfaceParam( hInterface, deviceName.c_str(), interfaceIndex, info, 0, pStringBuffer.get(), &stringSize );
	if( result != 0 )
	{
		WriteLogMessage( wxString::Format( wxT("ERROR during call to TLIMV_TLGetDeviceInterfaceParam( %p, %s, %d, %d, 0, %p, %p ): %d.\n"), hInterface, deviceName.c_str(), interfaceIndex, info, pStringBuffer.get(), &stringSize, result ), m_ERROR_STYLE );
		return string("");
	}
	return string(pStringBuffer.get());
}

//-----------------------------------------------------------------------------
string IPConfigureFrame::GetDeviceStringInfo( MVTLI_INTERFACE_HANDLE hInterface, const string& deviceName, DEVICE_INFO_CMD info )
//-----------------------------------------------------------------------------
{
	size_t stringSize = 0;
	int result = TLGetDeviceInfo( hInterface, deviceName.c_str(), info, 0, 0, &stringSize );
	if( result != 0 )
	{
		WriteLogMessage( wxString::Format( wxT("ERROR during call to TLGetDeviceInfo( %p, %s, %d, 0, 0, %p ): %d.\n"), hInterface, deviceName.c_str(), info, &stringSize, result ), m_ERROR_STYLE );
		return string("");
	}
	auto_array_ptr<char> pStringBuffer(stringSize);
	result = TLGetDeviceInfo( hInterface, deviceName.c_str(), info, 0, pStringBuffer.get(), &stringSize );
	if( result != 0 )
	{
		WriteLogMessage( wxString::Format( wxT("ERROR during call to TLGetDeviceInfo( %p, %s, %d, 0, %p, %p ): %d.\n"), hInterface, deviceName.c_str(), info, pStringBuffer.get(), &stringSize, result ), m_ERROR_STYLE );
		return string("");
	}
	return string(pStringBuffer.get());
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnAbout(wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxBoxSizer *pTopDownSizer;
	wxDialog dlg(this, wxID_ANY, wxString(_("About mvIPConfigure")));
	wxIcon icon(mvIcon_xpm);
	dlg.SetIcon( icon );
	pTopDownSizer = new wxBoxSizer(wxVERTICAL);
	wxStaticText *pText = new wxStaticText( &dlg, wxID_ANY, wxT("configure tool for GigE Vision (tm) devices") );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("(C) 2008 - %s by %s"), CURRENT_YEAR, COMPANY_NAME ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("Version %s"), VERSION_STRING ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxT("Support contact: www.matrix-vision.de") );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("This tool has been written using wxWidgets (www.wxwidgets.org) and was compiled with version %d.%d.%d of this library"), wxMAJOR_VERSION, wxMINOR_VERSION, wxMINOR_VERSION ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	pText = new wxStaticText( &dlg, wxID_ANY, wxString::Format( wxT("The complete source of this application can be obtained by contacting %s"), COMPANY_NAME ) );
	pTopDownSizer->Add( pText, 0, wxALL | wxALIGN_CENTER, 5 );
	wxButton *pBtnOK = new wxButton(&dlg, wxID_OK, wxT("OK"));
	pBtnOK->SetDefault();
	pTopDownSizer->Add( pBtnOK, 0, wxALL | wxALIGN_RIGHT, 15 );
	dlg.SetSizer( pTopDownSizer );
	pTopDownSizer->Fit( &dlg );
	dlg.ShowModal();
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnAssignTemporaryIP( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	AssignIPDlg dlg(this, m_hTLI);
	if( dlg.ShowModal() == wxID_OK )
	{
		UpdateDeviceList();
	}
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnBtnApplyChanges( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	int currentItem = m_pDevListCtrl->GetCurrentItemIndex();
	if( currentItem < 0 )
	{
		WriteLogMessage( wxT("ERROR: No device selected.\n"), m_ERROR_STYLE );
		return;
	}

	wxString itemText(m_pDevListCtrl->GetItemText( currentItem ));
	wxListItem info;
	info.m_itemId = currentItem;
	info.m_col = lcSerial;
	info.m_mask = wxLIST_MASK_TEXT;
	if( !m_pDevListCtrl->GetItem( info ) )
	{
		WriteLogMessage( wxString::Format( wxT("ERROR: Could not obtain serial number for device %s.\n"), itemText.c_str() ) );
		return;
	}

	WriteLogMessage( wxString::Format( wxT("Trying to establish write access to device %s(%s).\n"), info.m_text.c_str(), itemText.c_str() ) );
	InterfaceContainer::const_iterator itInterface = m_TLIInterfaces.find( string(m_pSTConnectedToIPAddress->GetLabel().mb_str()) );
	if( itInterface == m_TLIInterfaces.end() )
	{
		WriteLogMessage( wxString::Format( wxT("ERROR: Could not obtain interface handle to adapter %s.\n"), m_pSTConnectedToIPAddress->GetLabel().c_str() ) );
		return;
	}

	DeviceMap::const_iterator itDev = m_devices.find( string(m_pSTSerialNumber->GetLabel().mb_str()) );
	if( itDev == m_devices.end() )
	{
		WriteLogMessage( wxString::Format( wxT("ERROR: Could not obtain device name for device %s on adapter %s.\n"), m_pSTSerialNumber->GetLabel().c_str(), m_pSTConnectedToIPAddress->GetLabel().c_str() ) );
		return;
	}

	int status = 0;
	MVTLI_DEVICE_HANDLE hDev = 0;
	LOGGED_TLI_CALL( TLOpenDevice, ( itInterface->second, itDev->second->deviceName_.c_str(), DEVICE_ACCESS_EXCLUSIVE, &hDev ), WriteLogMessage )
	if( status != 0 )
	{
		return;
	}

	unsigned int timeout_ms = 2000;
	LOGGED_TLI_CALL( TLIMV_DevSetParam, ( hDev, DEVICE_INFO_GVCP_MESSAGE_TIMEOUT, &timeout_ms, sizeof(timeout_ms) ), WriteLogMessage )

	WriteLogMessage( wxString::Format( wxT("Trying to apply changes to device %s(%s).\n"), info.m_text.c_str(), itemText.c_str() ) );
	for( unsigned int i=0; i<itDev->second->interfaceCount_; i++ )
	{
		if( itDev->second->interfaceInfo_[i].supportsDHCP_ )
		{
			LOGGED_TLI_CALL( TLIMV_DevSetInterfaceParam, ( hDev, i, DEVICE_INFO_CURRENT_IP_DHCP, &m_interfaceInfo[i].DHCPEnabled_, sizeof(m_interfaceInfo[i].DHCPEnabled_) ), WriteLogMessage )
		}

		if( itDev->second->interfaceInfo_[i].supportsPersistentIP_ )
		{
			LOGGED_TLI_CALL( TLIMV_DevSetInterfaceParam, ( hDev, i, DEVICE_INFO_CURRENT_IP_PERSISTENT, &m_interfaceInfo[i].persistentIPEnabled_, sizeof(m_interfaceInfo[i].persistentIPEnabled_) ), WriteLogMessage )
			LOGGED_TLI_CALL( TLIMV_DevSetInterfaceParam, ( hDev, i, DEVICE_INFO_PERSISTENT_IP_STRING, m_interfaceInfo[i].persistentIPAddress_.c_str(), m_interfaceInfo[i].persistentIPAddress_.length() ), WriteLogMessage )
			LOGGED_TLI_CALL( TLIMV_DevSetInterfaceParam, ( hDev, i, DEVICE_INFO_PERSISTENT_NETMASK_STRING, m_interfaceInfo[i].persistentSubnetMask_.c_str(), m_interfaceInfo[i].persistentSubnetMask_.length() ), WriteLogMessage )
			LOGGED_TLI_CALL( TLIMV_DevSetInterfaceParam, ( hDev, i, DEVICE_INFO_PERSISTENT_DEFAULT_GATEWAY_STRING, m_interfaceInfo[i].persistentDefaultGateway_.c_str(), m_interfaceInfo[i].persistentDefaultGateway_.length() ), WriteLogMessage )
		}
	}
	if( itDev->second->supportsUserDefinedName_ )
	{
		LOGGED_TLI_CALL( TLIMV_DevSetParam, ( hDev, DEVICE_INFO_USERNAME, m_pTCUserDefinedName->GetValue().mb_str(), m_pTCUserDefinedName->GetValue().Length() ), WriteLogMessage )
	}

	WriteLogMessage( wxString::Format( wxT("Trying to close device %s(%s).\n"), info.m_text.c_str(), itemText.c_str() ) );
	LOGGED_TLI_CALL( DevClose, ( hDev ), WriteLogMessage )
	WriteLogMessage( wxT("Done...\n") );
	UpdateDeviceList();
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnBtnConfigure( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	UpdateDlgControls( true );
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnCBUseDHCP( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	m_interfaceInfo[m_pSCInterfaceSelector->GetValue()].DHCPEnabled_ = m_pCBUseDHCP->GetValue();
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnCBUsePersistentIP( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	m_interfaceInfo[m_pSCInterfaceSelector->GetValue()].persistentIPEnabled_ = m_pCBUsePersistentIP->GetValue();
	m_pTCPersistentIPAddress->Enable( m_pCBUsePersistentIP->GetValue() );
	m_pTCPersistentSubnetMask->Enable( m_pCBUsePersistentIP->GetValue() );
	m_pTCPersistentDefaultGateway->Enable( m_pCBUsePersistentIP->GetValue() );
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnListItemDeselected( int /*deviceIndex*/ )
//-----------------------------------------------------------------------------
{
	UpdateDlgControls( false );
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnListItemSelected( int /*deviceIndex*/ )
//-----------------------------------------------------------------------------
{
	UpdateDlgControls( false );
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnInterfaceSelectorTextChanged( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	WriteLogMessage( wxString::Format(wxT("Interface %d selected\n"), m_pSCInterfaceSelector->GetValue() ) );
	UpdateDlgControls( true );
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnPersistentIPTextChanged( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	if( m_pTCPersistentIPAddress )
	{
		m_interfaceInfo[m_pSCInterfaceSelector->GetValue()].persistentIPAddress_ = m_pTCPersistentIPAddress->GetValue().mb_str();
	}
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnPersistentNetmaskTextChanged( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	if( m_pTCPersistentSubnetMask )
	{
		m_interfaceInfo[m_pSCInterfaceSelector->GetValue()].persistentSubnetMask_ = m_pTCPersistentSubnetMask->GetValue().mb_str();
	}
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnPersistentGatewayTextChanged( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	if( m_pTCPersistentDefaultGateway )
	{
		m_interfaceInfo[m_pSCInterfaceSelector->GetValue()].persistentDefaultGateway_ = m_pTCPersistentDefaultGateway->GetValue().mb_str();
	}
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnQuit( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	// true is to force the frame to close
	Close( true );
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::OnUpdateDeviceList( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	UpdateDeviceList();
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::UpdateDeviceList( void )
//-----------------------------------------------------------------------------
{
	WriteLogMessage( wxString(wxT("Updating device list...\n")) );
	int status = 0;
	char hasChanged = 0;
	LOGGED_TLI_CALL( TLUpdateInterfaceList, ( m_hTLI, &hasChanged ), WriteLogMessage );
	unsigned int interfaceCnt = 0;
	LOGGED_TLI_CALL( TLGetNumInterfaces, ( m_hTLI, &interfaceCnt ), WriteLogMessage );
	if( status != 0 )
	{
		return;
	}

	if( interfaceCnt < 1 )
	{
		WriteLogMessage( wxT("No interfaces detected.\n"), m_ERROR_STYLE );
		return;
	}

	WriteLogMessage( wxString::Format( wxT("%d interface%s detected.\n"), interfaceCnt, ( interfaceCnt > 1 ) ? wxT("s") : wxT("") ) );

	InterfaceContainer lastInterfaceList(m_TLIInterfaces); // after this loop, this list will contain all the interfaces, that have disappeared...
	for( unsigned int i=0; i<interfaceCnt; i++ )
	{
		size_t stringSize = 0;
		status = TLGetInterfaceName( m_hTLI, i, 0, &stringSize );
		if( status != 0 )
		{
			WriteLogMessage( wxString::Format( wxT("ERROR during call to TLGetInterfaceName( %p, %d, 0, %p ).\n"), m_hTLI, i, &stringSize ), m_ERROR_STYLE );
			continue;
		}
		auto_array_ptr<char> pStringBuffer(stringSize);
		status = TLGetInterfaceName( m_hTLI, i, pStringBuffer.get(), &stringSize );
		if( status != 0 )
		{
			WriteLogMessage( wxString::Format( wxT("ERROR during call to TLGetInterfaceName( %p, %d, %p, %p ).\n"), m_hTLI, i, pStringBuffer.get(), &stringSize ), m_ERROR_STYLE );
			continue;
		}
		if( m_TLIInterfaces.find( string(pStringBuffer.get() ) ) == m_TLIInterfaces.end() )
		{
			// this is a new interface
			MVTLI_INTERFACE_HANDLE hInterface = 0;
			status = TLOpenInterface( m_hTLI, pStringBuffer.get(), &hInterface );
			if( status != 0 )
			{
				WriteLogMessage( wxString::Format( wxT("ERROR during call to TLOpenInterface( %p, %p, %p ).\n"), m_hTLI, pStringBuffer.get(), &hInterface ), m_ERROR_STYLE );
				continue;
			}
			m_TLIInterfaces.insert( make_pair( string(pStringBuffer.get()), hInterface ) );
		}
		else
		{
			// this interface should still be there
			InterfaceContainer::iterator it = lastInterfaceList.find( string(pStringBuffer.get() ) );
			assert( ( it != lastInterfaceList.end() ) && "BUG detected in interface handling. If this interface is missing in the list of interfaces detected last time there is a bug in the implementation" );
			if( it != lastInterfaceList.end() )
			{
				lastInterfaceList.erase( it );
			}
		}
	}
	InterfaceContainer::iterator itInterface = m_TLIInterfaces.begin();
	InterfaceContainer::iterator itInterfaceEND = m_TLIInterfaces.end();
	while( itInterface != itInterfaceEND )
	{
		InterfaceContainer::iterator itLast = lastInterfaceList.find( itInterface->first );
		if( itLast != lastInterfaceList.end() )
		{
			// this interface is gone now...
			/// \todo close it?
			m_TLIInterfaces.erase( itInterface );
			itInterface = m_TLIInterfaces.begin();
		}
		else
		{
			++itInterface;
		}
	}

	for_each( m_devices.begin(), m_devices.end(), ptr_fun(DeleteSecond<const string, DetecedDeviceInfo*>) );
	m_devices.clear();
	InterfaceContainer::iterator itInterfaces = m_TLIInterfaces.begin();
	InterfaceContainer::iterator itInterfacesEnd = m_TLIInterfaces.end();
	while( itInterfaces != itInterfacesEnd )
	{
		// each device is supposed to answer within 1 second
		LOGGED_TLI_CALL( TLUpdateDeviceList, ( itInterfaces->second, 1250 ), WriteLogMessage )
		unsigned int deviceCnt = 0;
		status = TLGetNumDevices( itInterfaces->second, &deviceCnt );
		if( status == 0 )
		{
			WriteLogMessage( wxString::Format( wxT("Interface %s reported %d device%s.\n"), ConvertedString( itInterfaces->first ).c_str(), deviceCnt, ( deviceCnt > 1 ) ? wxT("s") : wxT("") ) );
			for( unsigned int i=0; i<deviceCnt; i++ )
			{
				size_t stringSize = 0;
				status = TLGetDeviceName( itInterfaces->second, i, 0, &stringSize );
				if( status != 0 )
				{
					WriteLogMessage( wxString::Format( wxT("ERROR during call to TLGetDeviceName( %p, %d, 0, %p ).\n"), itInterfaces->second, i, &stringSize ), m_ERROR_STYLE );
					continue;
				}

				auto_array_ptr<char> pStringBuffer(stringSize);
				status = TLGetDeviceName( itInterfaces->second, i, pStringBuffer.get(), &stringSize );
				if( status != 0 )
				{
					WriteLogMessage( wxString::Format( wxT("ERROR during call to TLGetDeviceName( %p, %d, %p, %p ).\n"), itInterfaces->second, i, pStringBuffer.get(), &stringSize ), m_ERROR_STYLE );
					continue;
				}

				string deviceName(pStringBuffer.get());
				string serial = GetDeviceStringInfo( itInterfaces->second, deviceName, DEVICE_INFO_SERIALNUMBER );
				DeviceMap::iterator itDev = m_devices.find( serial );
				if( itDev == m_devices.end() )
				{
					// this is the first time this device has been located in this 'Enumerate' run. It might 
					// however be found again at a different network adapter

					string model(GetDeviceStringInfo( itInterfaces->second, deviceName, DEVICE_INFO_MODEL ));
					string manufacturer(GetDeviceStringInfo( itInterfaces->second, deviceName, DEVICE_INFO_MANUFACTURER ));
					string userDefinedName(GetDeviceStringInfo( itInterfaces->second, deviceName, DEVICE_INFO_USERNAME ));
					string serial(GetDeviceStringInfo( itInterfaces->second, deviceName, DEVICE_INFO_SERIALNUMBER ));
					unsigned int interfaceCount = 1;
					size_t bufferSize = sizeof(interfaceCount);
					LOGGED_TLI_CALL( TLGetDeviceInfo, ( itInterfaces->second, deviceName.c_str(), DEVICE_INFO_INTERFACE_COUNT, 0, &interfaceCount, &bufferSize ), WriteLogMessage )
					if( interfaceCount > DetecedDeviceInfo::MAX_INTERFACE_COUNT )
					{
						WriteLogMessage( wxString::Format( wxT("ERROR!!! This device claims to support %u interfaces, while the current version of the standard only allows %lu interfaces.\n"), interfaceCount, (long unsigned int) DetecedDeviceInfo::MAX_INTERFACE_COUNT ), m_ERROR_STYLE );
						interfaceCount = 4;
					}
					unsigned char supportsUserDefinedName = 0;
					bufferSize = sizeof(supportsUserDefinedName);
					LOGGED_TLI_CALL( TLGetDeviceInfo, ( itInterfaces->second, deviceName.c_str(), DEVICE_INFO_SUPPORT_USER_DEFINED_NAME, 0, &supportsUserDefinedName, &bufferSize ), WriteLogMessage )
					DetecedDeviceInfo* p = new DetecedDeviceInfo(deviceName, serial, model, manufacturer, userDefinedName, supportsUserDefinedName, itInterfaces->first, interfaceCount);
					pair<DeviceMap::iterator, bool> insertPair = m_devices.insert( make_pair( serial, p ) );
					if( !insertPair.second )
					{
						WriteLogMessage( wxT("Internal ERROR!!! This device has been inserted already.\n"), m_ERROR_STYLE );
					}
					else
					{
						WriteLogMessage( wxString::Format( wxT("Device %s detected on interface %s).\n"), ConvertedString(serial).c_str(), ConvertedString(insertPair.first->second->adapterIPAddress_).c_str() ) );
						for( unsigned int i=0; i<interfaceCount; i++ )
						{
							InterfaceInfo& info = insertPair.first->second->interfaceInfo_[i];
							info.currentIPAddress_ = GetDeviceInterfaceStringInfo( itInterfaces->second, deviceName, i, DEVICE_INFO_IP_STRING );
							info.currentSubnetMask_ = GetDeviceInterfaceStringInfo( itInterfaces->second, deviceName, i, DEVICE_INFO_CURRENT_NETMASK_STRING );
							info.currentDefaultGateway_ = GetDeviceInterfaceStringInfo( itInterfaces->second, deviceName, i, DEVICE_INFO_CURRENT_DEFAULT_GATEWAY_STRING );
							info.persistentIPAddress_ = GetDeviceInterfaceStringInfo( itInterfaces->second, deviceName, i, DEVICE_INFO_PERSISTENT_IP_STRING );
							info.persistentSubnetMask_ = GetDeviceInterfaceStringInfo( itInterfaces->second, deviceName, i, DEVICE_INFO_PERSISTENT_NETMASK_STRING );
							info.persistentDefaultGateway_ = GetDeviceInterfaceStringInfo( itInterfaces->second, deviceName, i, DEVICE_INFO_PERSISTENT_DEFAULT_GATEWAY_STRING );
							info.MACAddress_ = GetDeviceInterfaceStringInfo( itInterfaces->second, deviceName, i, DEVICE_INFO_MAC_STRING );
							bufferSize = sizeof(info.supportsLLA_);
							LOGGED_TLI_CALL( TLIMV_TLGetDeviceInterfaceParam, ( itInterfaces->second, deviceName.c_str(), i, DEVICE_INFO_SUPPORT_IP_LLA, 0, &info.supportsLLA_, &bufferSize ), WriteLogMessage )
							bufferSize = sizeof(info.supportsDHCP_);
							LOGGED_TLI_CALL( TLIMV_TLGetDeviceInterfaceParam, ( itInterfaces->second, deviceName.c_str(), i, DEVICE_INFO_SUPPORT_IP_DHCP, 0, &info.supportsDHCP_, &bufferSize ), WriteLogMessage )
							bufferSize = sizeof(info.supportsPersistentIP_);
							LOGGED_TLI_CALL( TLIMV_TLGetDeviceInterfaceParam, ( itInterfaces->second, deviceName.c_str(), i, DEVICE_INFO_SUPPORT_IP_PERSISTENT, 0, &info.supportsPersistentIP_, &bufferSize ), WriteLogMessage )
							bufferSize = sizeof(info.LLAEnabled_);
							LOGGED_TLI_CALL( TLIMV_TLGetDeviceInterfaceParam, ( itInterfaces->second, deviceName.c_str(), i, DEVICE_INFO_CURRENT_IP_LLA, 0, &info.LLAEnabled_, &bufferSize ), WriteLogMessage )
							bufferSize = sizeof(info.DHCPEnabled_);
							LOGGED_TLI_CALL( TLIMV_TLGetDeviceInterfaceParam, ( itInterfaces->second, deviceName.c_str(), i, DEVICE_INFO_CURRENT_IP_DHCP, 0, &info.DHCPEnabled_, &bufferSize ), WriteLogMessage )
							bufferSize = sizeof(info.persistentIPEnabled_);
							LOGGED_TLI_CALL( TLIMV_TLGetDeviceInterfaceParam, ( itInterfaces->second, deviceName.c_str(), i, DEVICE_INFO_CURRENT_IP_PERSISTENT, 0, &info.persistentIPEnabled_, &bufferSize ), WriteLogMessage )
						}
					}
				}
				else
				{
					WriteLogMessage( wxString::Format( wxT("Device %s (reachable via interface %s) can also be reached via interface %s.\n"), ConvertedString(serial).c_str(), ConvertedString(itDev->second->adapterIPAddress_).c_str(), ConvertedString(itInterfaces->first).c_str() ) );
				}
			}
		}
		else
		{
			WriteLogMessage( wxString::Format( wxT("ERROR during call to TLGetNumDevices( %p, %p ).\n"), itInterfaces->second, &deviceCnt ), m_ERROR_STYLE );
		}
		++itInterfaces;
	}
	BuildList();
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::UpdateDlgControls( bool boEdit )
//-----------------------------------------------------------------------------
{
	int currentItem = m_pDevListCtrl->GetCurrentItemIndex();
	wxListItem info;
	if( currentItem >= 0 )
	{
		info.m_itemId = currentItem;
		info.m_col = lcSerial;
		info.m_mask = wxLIST_MASK_TEXT;
		if( !m_pDevListCtrl->GetItem( info ) )
		{
			return;
		}
	}

	DeviceMap::const_iterator it = m_devices.find( string(info.m_text.mb_str()) );
	bool boDeviceValid = ( it != m_devices.end() );
	if( boDeviceValid )
	{
		for( unsigned int i=0; i<DetecedDeviceInfo::MAX_INTERFACE_COUNT; i++ )
		{
			m_interfaceInfo[0] = it->second->interfaceInfo_[0];
		}
	}
	else
	{
		boEdit = false;
	}

	// device info controls
	m_pSTManufacturer->SetLabel( ConvertedString(boDeviceValid ? it->second->manufacturer_.c_str() : "-") );
	m_pSTSerialNumber->SetLabel( ConvertedString(boDeviceValid ? it->second->deviceSerial_.c_str() : "-") );
	m_pTCUserDefinedName->SetValue( ConvertedString(boDeviceValid ? it->second->userDefinedName_.c_str() : "-") );
	m_pTCUserDefinedName->Enable( boEdit && it->second->supportsUserDefinedName_ );
	m_pSTInterfaceCount->SetLabel( boDeviceValid ? wxString::Format( wxT("%d"), it->second->interfaceCount_ ) : wxT("-") );

	// interface selector
	m_pSCInterfaceSelector->Enable( boEdit );
	m_pSCInterfaceSelector->SetRange( 0, ( boDeviceValid ? it->second->interfaceCount_ - 1 : 0 ) );
	if( m_pSCInterfaceSelector->GetValue() > m_pSCInterfaceSelector->GetMax() )
	{
		m_pSCInterfaceSelector->SetValue( 0 );
	}

	int interfaceIndex = m_pSCInterfaceSelector->GetValue();

	// current IP controls
	m_pSTCurrentIPAddress->SetLabel( ConvertedString(boDeviceValid ? m_interfaceInfo[interfaceIndex].currentIPAddress_.c_str() : "-") );
	m_pSTCurrentSubnetMask->SetLabel( ConvertedString(boDeviceValid ? m_interfaceInfo[interfaceIndex].currentSubnetMask_.c_str() : "-") );
	m_pSTCurrentDefaultGateway->SetLabel( ConvertedString(boDeviceValid ? m_interfaceInfo[interfaceIndex].currentDefaultGateway_.c_str() : "-") );
	m_pSTMACAddress->SetLabel( ConvertedString(boDeviceValid ? m_interfaceInfo[interfaceIndex].MACAddress_.c_str() : "-") );
	m_pSTConnectedToIPAddress->SetLabel( ConvertedString(boDeviceValid ? it->second->adapterIPAddress_.c_str() : "-") );

	// persistent IP controls
	m_pTCPersistentIPAddress->SetValue( ConvertedString((boDeviceValid && m_interfaceInfo[interfaceIndex].supportsPersistentIP_) ? m_interfaceInfo[interfaceIndex].persistentIPAddress_.c_str() : "-") );
	m_pTCPersistentIPAddress->Enable( boEdit && m_interfaceInfo[interfaceIndex].supportsPersistentIP_ && m_interfaceInfo[interfaceIndex].persistentIPEnabled_ );
	m_pTCPersistentSubnetMask->SetValue( ConvertedString((boDeviceValid && m_interfaceInfo[interfaceIndex].supportsPersistentIP_) ? m_interfaceInfo[interfaceIndex].persistentSubnetMask_.c_str() : "-") );
	m_pTCPersistentSubnetMask->Enable( boEdit && m_interfaceInfo[interfaceIndex].supportsPersistentIP_ && m_interfaceInfo[interfaceIndex].persistentIPEnabled_ );
	m_pTCPersistentDefaultGateway->SetValue( ConvertedString((boDeviceValid && m_interfaceInfo[interfaceIndex].supportsPersistentIP_) ? m_interfaceInfo[interfaceIndex].persistentDefaultGateway_.c_str() : "-") );
	m_pTCPersistentDefaultGateway->Enable( boEdit && m_interfaceInfo[interfaceIndex].supportsPersistentIP_ && m_interfaceInfo[interfaceIndex].persistentIPEnabled_ );

	// IP configuration controls
	m_pCBUsePersistentIP->Enable( boEdit && m_interfaceInfo[interfaceIndex].supportsPersistentIP_ );
	m_pCBUsePersistentIP->SetValue( boDeviceValid && m_interfaceInfo[interfaceIndex].persistentIPEnabled_ );
	m_pCBUseDHCP->Enable( boEdit && m_interfaceInfo[interfaceIndex].supportsDHCP_ );
	m_pCBUseDHCP->SetValue( boDeviceValid && m_interfaceInfo[interfaceIndex].DHCPEnabled_ );
	// never enable this check box as this feature must always be active anyway
	m_pCBUseLLA->SetValue( boDeviceValid && m_interfaceInfo[interfaceIndex].LLAEnabled_ );

	// buttons
	m_pBtnConfigure->Enable( boDeviceValid && !boEdit );
	m_pBtnApplyChanges->Enable( boEdit );
}

//-----------------------------------------------------------------------------
void IPConfigureFrame::WriteLogMessage( const wxString& msg, const wxTextAttr& style /* = wxTextAttr(wxColour(0, 0, 0)) */ )
//-----------------------------------------------------------------------------
{
	if( m_pLogWindow )
	{
		long posBefore = m_pLogWindow->GetLastPosition();
		m_pLogWindow->WriteText( msg );
		long posAfter = m_pLogWindow->GetLastPosition();
		m_pLogWindow->SetStyle( posBefore, posAfter, style );
	}
}
